from yafowil.base import factory
from yafowil.utils import entry_point
import os
import webresource as wr


resources_dir = os.path.join(os.path.dirname(__file__), 'resources')


##############################################################################
# Default
##############################################################################

# webresource ################################################################

resources = wr.ResourceGroup(
    name='yafowil.widget.ace',
    directory=resources_dir,
    path='yafowil-ace'
)
resources.add(wr.ScriptResource(
    name='ace-js',
    directory=os.path.join(resources_dir, 'ace'),
    path='yafowil-ace/ace',
    resource='ace.js'
))
resources.add(wr.ScriptResource(
    name='yafowil-ace-js',
    depends=[
        'jquery-js',
        'ace-js'
    ],
    resource='widget.js',
    compressed='widget.min.js'
))
resources.add(wr.StyleResource(
    name='yafowil-ace-css',
    directory=resources_dir,
    resource='widget.css'
))

# B/C resources ##############################################################

js = [{
    'group': 'yafowil.widget.ace.dependencies',
    'resource': 'ace/ace.js',
    'order': 20,
}, {
    'group': 'yafowil.widget.ace.common',
    'resource': 'widget.js',
    'order': 21,
}]
css = [{
    'group': 'yafowil.widget.ace.common',
    'resource': 'widget.css',
    'order': 20,
}]


##############################################################################
# Registration
##############################################################################

@entry_point(order=10)
def register():
    from yafowil.widget.ace import widget  # noqa

    widget_name = 'yafowil.widget.ace'

    # Default
    factory.register_theme(
        'default',
        widget_name,
        resources_dir,
        js=js,
        css=css
    )
    factory.register_resources('default', widget_name, resources)



from yafowil.base import factory


DOC_ACE_PYTHON = """
ACE in Python mode
------------------

Set the 'mode' option to 'python' to create a Python Ace widget.

.. code-block:: python

    value = '''
    def foo():
        return 'foo'
    '''
    ace = factory('#field:ace', value=value, props={
        'label': 'Python source',
        'required': 'Code is required',
        'theme': 'github',
        'mode': 'python',
    })
"""

ace_py_value = """\
def foo():
    return 'foo'
"""


def ace_python():
    part = factory(u'fieldset', name='yafowil.widget.ace.python')
    part['ace'] = factory('#field:ace', value=ace_py_value, props={
        'label': 'Python source',
        'required': 'Code is required',
        'theme': 'github',
        'mode': 'python',
    })
    return {
        'widget': part,
        'doc': DOC_ACE_PYTHON,
        'title': 'Python',
    }


DOC_ACE_JS = """
ACE in Javascript mode
----------------------

Set the 'mode' option to 'javascript' to create a JavaScript Ace widget.

.. code-block:: python

    value = '''
    var foo = function() {
        return 'foo';
    };
    '''
    ace = factory('#field:ace', value=value, props={
        'label': 'Javascript Source',
        'required': 'Code is required',
        'theme': 'github',
        'mode': 'javascript',
    })
"""

ace_js_value = """\
var foo = function() {
    return 'foo';
};
"""


def ace_js():
    part = factory(u'fieldset', name='yafowil.widget.ace.js')
    part['ace'] = factory('#field:ace', value=ace_js_value, props={
        'label': 'Javascript Source',
        'required': 'Code is required',
        'theme': 'github',
        'mode': 'javascript',
    })
    return {
        'widget': part,
        'doc': DOC_ACE_JS,
        'title': 'Javascript',
    }


DOC_ACE_THEMES = """
Themes
------

Customize syntax highlighting and appearance with one of the available ACE themes.

.. code-block:: python

    value = '''
    function foo(items) {
        for (var i=0; i<=items.length; i++) {
            alert(items[i]);
        }
    }
    '''
    ace = factory('#field:ace', value=value, props={
        'label': 'Cobalt Theme',
        'required': 'Code is required',
        'theme': 'cobalt',
        'mode': 'javascript'
    })
"""

ace_theme_value = """\
function foo(items) {
    for (var i=0; i<=items.length; i++) {
        alert(items[i]);
    }
}
"""


def ace_themes():
    part = factory(u'fieldset', name='yafowil.widget.ace.themes')
    part['ace'] = factory('#field:ace', value=ace_theme_value, props={
        'label': 'Cobalt Theme',
        'required': 'Code is required',
        'theme': 'cobalt',
        'mode': 'javascript'
    })
    return {
        'widget': part,
        'doc': DOC_ACE_THEMES,
        'title': 'Themes',
    }


def get_example():
    return [ace_python(), ace_js(), ace_themes()]



from yafowil.base import factory
from yafowil.base import fetch_value
from yafowil.common import generic_extractor
from yafowil.common import generic_required_extractor
from yafowil.utils import as_data_attrs
from yafowil.utils import attr_value
from yafowil.utils import cssid
from yafowil.utils import managedprops


@managedprops('basepath', 'theme', 'mode')
def ace_edit_renderer(widget, data):
    value = fetch_value(widget, data)
    if not value:
        value = ''
    ta_attrs = {
        'id': cssid(widget, 'ace', 'value'),
        'name': widget.dottedpath,
        'class': 'ace-editor-value',
        'style': 'display:none;',
    }
    ta = data.tag('textarea', value, **ta_attrs)
    editor_attrs = {
        'id': cssid(widget, 'ace'),
        'class': 'ace-editor',
    }
    editor = data.tag('div', value, **editor_attrs)
    wrapper_attrs = {
        'class': 'ace-editor-wrapper',
    }
    wrapper_attrs.update(as_data_attrs({
        'yafowil-ace': {
            'basepath': attr_value('basepath', widget, data),
            'theme': attr_value('theme', widget, data),
            'mode': attr_value('mode', widget, data)
        }
    }))
    return data.tag('div', ta + editor, **wrapper_attrs)


def ace_display_renderer(widget, data):
    raise NotImplementedError(
        u"``yafowil.widget.ace`` does not support display mode yet"
    )


factory.register(
    'ace',
    extractors=[
        generic_extractor,
        generic_required_extractor
    ],
    edit_renderers=[ace_edit_renderer],
    display_renderers=[ace_display_renderer]
)

factory.doc['blueprint']['ace'] = """\
Add-on blueprint `yafowil.widget.ace
<http://github.com/conestack/yafowil.widget.ace/>`_ .
"""

factory.defaults['ace.default'] = ''

factory.defaults['ace.class'] = 'ace_editor'

factory.defaults['ace.basepath'] = ''
factory.doc['props']['ace.basepath'] = """\
ACE Basepath for resources.
"""

factory.defaults['ace.theme'] = 'github'
factory.doc['props']['ace.theme'] = """\
ACE Theme.
"""

factory.defaults['ace.mode'] = 'python'
factory.doc['props']['ace.mode'] = """\
ACE Mode.
"""



from node.utils import UNSET
from yafowil.base import ExtractionError
from yafowil.base import factory
from yafowil.compat import IS_PY2
from yafowil.tests import fxml
from yafowil.tests import YafowilTestCase
import os
import unittest


if not IS_PY2:
    from importlib import reload


def np(path):
    return path.replace('/', os.path.sep)


class TestACEWidget(YafowilTestCase):

    def setUp(self):
        super(TestACEWidget, self).setUp()
        from yafowil.widget import ace
        from yafowil.widget.ace import widget
        reload(widget)
        ace.register()

    def test_edit_renderer(self):
        widget = factory('ace', 'acefield', props={'required': True})
        self.checkOutput("""
        <div class="ace-editor-wrapper"
             data-yafowil-ace="{&quot;basepath&quot;:
                                &quot;&quot;,
                                &quot;theme&quot;:
                                &quot;github&quot;,
                                &quot;mode&quot;:
                                &quot;python&quot;}">
          <textarea class="ace-editor-value"
                    id="ace-acefield-value"
                    name="acefield"
                    style="display:none;"/>
          <div class="ace-editor" id="ace-acefield"/>
        </div>
        """, fxml(widget()))

    def test_display_renderer(self):
        value = 'class Foo(object): pass'
        widget = factory('ace', 'acefield', value=value, mode='display')
        with self.assertRaises(NotImplementedError) as arc:
            widget()
        msg = '``yafowil.widget.ace`` does not support display mode yet'
        self.assertEqual(str(arc.exception), msg)

    def test_extraction(self):
        widget = factory('ace', 'acefield', props={'required': True})
        request = {'acefield': ''}
        data = widget.extract(request)
        self.assertEqual(data.name, 'acefield')
        self.assertEqual(data.value, UNSET)
        self.assertEqual(data.extracted, '')
        self.assertEqual(data.errors, [
            ExtractionError('Mandatory field was empty')
        ])

        request = {'acefield': 'class Foo(object): pass'}
        data = widget.extract(request)
        self.assertEqual(data.name, 'acefield')
        self.assertEqual(data.value, UNSET)
        self.assertEqual(data.extracted, 'class Foo(object): pass')
        self.assertEqual(data.errors, [])

        self.checkOutput("""
        <div class="ace-editor-wrapper"
             data-yafowil-ace="{&quot;basepath&quot;:
                                &quot;&quot;,
                                &quot;theme&quot;:
                                &quot;github&quot;,
                                &quot;mode&quot;:
                                &quot;python&quot;}">
          <textarea class="ace-editor-value"
                    id="ace-acefield-value"
                    name="acefield"
                    style="display:none;">class Foo(object): pass</textarea>
          <div class="ace-editor"
               id="ace-acefield">class Foo(object): pass</div>
        </div>
        """, fxml(widget(data)))

    def test_resources(self):
        factory.theme = 'default'
        resources = factory.get_resources('yafowil.widget.ace')
        self.assertTrue(resources.directory.endswith(np('/ace/resources')))
        self.assertEqual(resources.name, 'yafowil.widget.ace')
        self.assertEqual(resources.path, 'yafowil-ace')

        scripts = resources.scripts
        self.assertEqual(len(scripts), 2)

        self.assertTrue(scripts[0].directory.endswith(np('/ace/resources/ace')))
        self.assertEqual(scripts[0].path, 'yafowil-ace/ace')
        self.assertEqual(scripts[0].file_name, 'ace.js')
        self.assertTrue(os.path.exists(scripts[0].file_path))

        self.assertTrue(scripts[1].directory.endswith(np('/ace/resources')))
        self.assertEqual(scripts[1].path, 'yafowil-ace')
        self.assertEqual(scripts[1].file_name, 'widget.min.js')
        self.assertTrue(os.path.exists(scripts[1].file_path))

        styles = resources.styles
        self.assertEqual(len(styles), 1)

        self.assertTrue(styles[0].directory.endswith(np('/ace/resources')))
        self.assertEqual(styles[0].path, 'yafowil-ace')
        self.assertEqual(styles[0].file_name, 'widget.css')
        self.assertTrue(os.path.exists(styles[0].file_path))


if __name__ == '__main__':
    unittest.main()



from setuptools import find_packages
from setuptools import setup
import os


def read_file(name):
    with open(os.path.join(os.path.dirname(__file__), name)) as f:
        return f.read()


version = '2.0a1'
shortdesc = 'ACE Editor Widget for YAFOWIL'
longdesc = '\n\n'.join([read_file(name) for name in [
    'README.rst',
    'CHANGES.rst',
    'LICENSE.rst'
]])


setup(
    name='yafowil.widget.ace',
    version=version,
    description=shortdesc,
    long_description=longdesc,
    classifiers=[
        'Environment :: Web Environment',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='',
    author='Yafowil Contributors',
    author_email='dev@conestack.org',
    url=u'http://github.com/conestack/yafowil.widget.ace',
    license='Simplified BSD',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['yafowil', 'yafowil.widget'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'yafowil>2.1.99',
    ],
    tests_require=[
        'lxml',
        'zope.testrunner'
    ],
    extras_require=dict(test=[
        'lxml',
        'zope.testrunner'
    ]),
    test_suite="yafowil.widget.ace.tests",
    entry_points="""
    [yafowil.plugin]
    register = yafowil.widget.ace:register
    example = yafowil.widget.ace.example:get_example
    """
)
